import 'dart:async';

import 'package:dropsource_app_utilities/dropsource_app_utilities.dart';
import 'package:firebase_crashlytics/firebase_crashlytics.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

enum AppEnvironment {
  development,
  staging,
  production,
  testing,
}

abstract class AppConfig<S> {
  AppConfig({
    @required this.appName,
    @required this.environment,
    @required this.apiBaseUrl,
    this.initializeCrashlytics = true,
    this.monitorConnectivity = true,
  });

  final String appName;
  final AppEnvironment environment;
  final String apiBaseUrl;
  final bool initializeCrashlytics;
  final bool monitorConnectivity;

  Future startCrashlytics() async {
    if (this.initializeCrashlytics) {
      FlutterError.onError = (FlutterErrorDetails details) {
        if (isInDebugMode()) {
          // In development mode simply print to console.
          FlutterError.dumpErrorToConsole(details);
        } else {
          // In production mode report to the application zone to report to
          // Crashlytics.
          Crashlytics.instance.onError(details);
        }
      };
    }
  }

  Future<S> loadState();

  Widget createApp(S state);

  Future run() async {
    await startCrashlytics();
    final _state = await loadState();
    runApp(createApp(_state));
  }
}
